//
//  AppDelegate.m
//  Socials
//
//  Created by Maarten Jansen on 02/08/12.
//  Copyright (c) 2012 iCapps. All rights reserved.
//

#import "AppDelegate.h"
#import "LoginViewController.h"
#import "HomeViewController.h"
#import <Accounts/Accounts.h>
#import <Twitter/Twitter.h>
#import <FacebookSDK/FacebookSDK.h>


#pragma mark - AppDelegate Extension
@interface AppDelegate()

@property (strong, nonatomic) UINavigationController *navigationController;

@end

#pragma mark - AppDelegate Implementation
@implementation AppDelegate
#pragma mark Synthesize
@synthesize window = _window;
@synthesize navigationController = _navigationController;

#pragma mark Lifecycle
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    self.window = [[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    // Override point for customization after application launch.
    LoginViewController *loginViewController = [[LoginViewController alloc] initWithNibName:@"LoginViewController" bundle:nil];
    self.navigationController = [[UINavigationController alloc] initWithRootViewController:loginViewController];
    self.navigationController.navigationBar.tintColor = [UIColor blackColor];
    self.navigationController.navigationBar.alpha = 0.7f;
    self.navigationController.navigationBar.translucent = YES;
    [self.navigationController setNavigationBarHidden:YES];
    self.window.rootViewController = self.navigationController;
    [self.window makeKeyAndVisible];
    [self restoreLastSessionIfExists];
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application {
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
    // Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    if (FBSession.activeSession.state == FBSessionStateCreatedOpening) {    
        [FBSession.activeSession close]; // so we close our session and start over  
    }
}

- (void)applicationWillTerminate:(UIApplication *)application {
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

#pragma mark - Other
- (void)restoreLastSessionIfExists {
    SocialAccountType lastActiveSocialAccountType = [[NSUserDefaults standardUserDefaults] integerForKey:kSocialAccountTypeKey];
    if(lastActiveSocialAccountType == SocialAccountTypeFacebook) {
        if (FBSession.activeSession.state == FBSessionStateCreatedTokenLoaded) {
            //The session has not expired so this will not create any visible UI activity.
            [self openFacebookSession];
        }
    } else if(lastActiveSocialAccountType == SocialAccountTypeTwitter) {
        [self getTwitterAccountOnCompletion:^(ACAccount *twitterAccount){
            //If we successfully retrieved a Twitter account
            if(twitterAccount) {
                //Make sure anything UI related happens on the main queue
                dispatch_async(dispatch_get_main_queue(), ^{
                    HomeViewController *homeViewController = [[HomeViewController alloc] initWithSocialAccountType:SocialAccountTypeTwitter];
                    [self.navigationController pushViewController:homeViewController animated:YES];
                });
            }
        }];
    }
}

#pragma mark - URL handler
- (BOOL)application:(UIApplication *)application 
            openURL:(NSURL *)url
  sourceApplication:(NSString *)sourceApplication 
         annotation:(id)annotation 
{
    return [FBSession.activeSession handleOpenURL:url]; 
}

#pragma mark - Twitter SDK
- (void)getTwitterAccountOnCompletion:(void (^)(ACAccount *))completionHandler {
    ACAccountStore *store = [[ACAccountStore alloc] init];
    ACAccountType *twitterType = [store accountTypeWithAccountTypeIdentifier:ACAccountTypeIdentifierTwitter];
    [store requestAccessToAccountsWithType:twitterType withCompletionHandler:^(BOOL granted, NSError *error) {
        if(granted) {
            // Remember that twitterType was instantiated above
            NSArray *twitterAccounts = [store accountsWithAccountType:twitterType];
            
            // If there are no accounts, we need to pop up an alert
            if(twitterAccounts == nil || [twitterAccounts count] == 0) {
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"No Twitter Accounts"
                                                                message:@"There are no Twitter accounts configured. You can add or create a Twitter account in Settings."
                                                               delegate:nil
                                                      cancelButtonTitle:@"OK"
                                                      otherButtonTitles:nil];
                [alert show];
            } else {
                //Get the first account in the array
                ACAccount *twitterAccount = [twitterAccounts objectAtIndex:0];
                //Save the used SocialAccountType so it can be retrieved the next time the app is started.
                [[NSUserDefaults standardUserDefaults] setInteger:SocialAccountTypeTwitter forKey:kSocialAccountTypeKey];
                [[NSUserDefaults standardUserDefaults] synchronize];
                //Call the completion handler so the calling object can retrieve the twitter account.
                completionHandler(twitterAccount);
            }
        }
    }];
}

#pragma mark - Facebook SDK
- (void)openFacebookSession {
    NSArray *facebookPermissions = [NSArray arrayWithObjects:@"user_about_me", @"user_status", nil];
    [FBSession sessionOpenWithPermissions:facebookPermissions completionHandler:
     ^(FBSession *session, FBSessionState state, NSError *error) {
         [self sessionStateChanged:session state:state error:error];
     }];
}

- (void)closeFacebookSession {
    [FBSession.activeSession closeAndClearTokenInformation];
}

- (void)sessionStateChanged:(FBSession *)session 
                      state:(FBSessionState) state
                      error:(NSError *)error {
    switch (state) {
        case FBSessionStateOpen: {
            //Save the used SocialAccountType so it can be retrieved the next time the app is started.
            [[NSUserDefaults standardUserDefaults] setInteger:SocialAccountTypeFacebook forKey:kSocialAccountTypeKey];
            [[NSUserDefaults standardUserDefaults] synchronize];
            //Go to the HomeViewController
            HomeViewController *homeViewController = [[HomeViewController alloc] initWithSocialAccountType:SocialAccountTypeFacebook];
            [self.navigationController pushViewController:homeViewController animated:YES];
        }
            break;
        case FBSessionStateClosed:
        case FBSessionStateClosedLoginFailed:
            // Once the user has logged out, we want them to be looking at the root view.
            [self.navigationController popToRootViewControllerAnimated:YES];            
            [FBSession.activeSession closeAndClearTokenInformation];
            break;
        default:
            break;
    }
    
    if (error) {
        UIAlertView *alertView = [[UIAlertView alloc]
                                  initWithTitle:@"Facebook Error"
                                  message:error.localizedDescription
                                  delegate:nil
                                  cancelButtonTitle:@"OK"
                                  otherButtonTitles:nil];
        [alertView show];
    }    
}

@end
