//
//  HomeViewController.m
//  Socials
//
//  Created by Maarten Jansen on 02/08/12.
//  Copyright (c) 2012 iCapps. All rights reserved.
//

#import "HomeViewController.h"
#import "AppDelegate.h"
#import "SVProgressHUD.h"
#import <FacebookSDK/FacebookSDK.h>
#import <Accounts/Accounts.h>
#import <Twitter/Twitter.h>
#import <QuartzCore/QuartzCore.h>

#pragma mark - HomeViewController Extension
@interface HomeViewController ()

/* IBOutlet */
@property (nonatomic, weak) IBOutlet UIImageView *socialLogoImageView;
@property (weak, nonatomic) IBOutlet UILabel *nameLabel;
@property (weak, nonatomic) IBOutlet UILabel *statusLabel;
@property (weak, nonatomic) IBOutlet UITextView *statusTextView;
@property (weak, nonatomic) IBOutlet UIView *userPhotoContainerView;


@property (nonatomic) SocialAccountType socialAccountType;
@property (atomic, strong) ACAccount *twitterAccount;
@property (atomic, strong) NSString *username;
@property (atomic, strong) NSString *name;
@property (atomic, strong) NSString *status;
@property (atomic, strong) UIView *userPhotoView;

@property (atomic) BOOL userDataLoaded;
@property (atomic) BOOL timelineDataLoaded;

@end

#pragma mark - HomeViewController Implementation
@implementation HomeViewController
#pragma mark Synthesize
@synthesize socialLogoImageView = _socialLogoImageView;
@synthesize nameLabel = _nameLabel;
@synthesize userPhotoContainerView = _userPhotoContainerView;
@synthesize statusLabel = _statusLabel;
@synthesize statusTextView = _statusTextView;
@synthesize socialAccountType = _socialAccountType;
@synthesize twitterAccount = _twitterAccount;
@synthesize username = _username;
@synthesize name = _name;
@synthesize status = _status;
@synthesize userPhotoView = _userPhotoView;
@synthesize userDataLoaded = _userDataLoaded;
@synthesize timelineDataLoaded = _timelineDataLoaded;

#pragma mark Lifecycle
- (id)initWithSocialAccountType:(SocialAccountType)socialAccountType {
    self = [super init];
    if(self) {
        self.socialAccountType = socialAccountType;
        self.username = nil;
        self.name =  nil;
        self.status = nil;
        self.userDataLoaded = NO;
        self.timelineDataLoaded = NO;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];   
    [[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleBlackTranslucent animated:NO];
    [self setupScreen];
}

- (void)viewDidUnload {
    [self setNameLabel: nil];
    [self setStatusTextView: nil];
    [self setSocialLogoImageView: nil];
    [self setUserPhotoContainerView:nil];
    [self setStatusLabel:nil];
    [self setStatusTextView:nil];
    [super viewDidUnload];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [SVProgressHUD showWithStatus:@"Loading..."];
    [self loadUserDetails];
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    [self.navigationController setNavigationBarHidden:NO animated:YES];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark Setup
- (void)setupScreen {
    //Add shadow to text in descriptionTextView
    self.statusTextView.layer.shadowColor = [UIColor darkTextColor].CGColor;
    self.statusTextView.layer.shadowOffset = CGSizeMake(0.0, -1.0);
    self.statusTextView.layer.shadowOpacity = 1.0f;
    self.statusTextView.layer.shadowRadius = 1.0f;
    
    //Setup the logout button
    self.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Logout" style:UIBarButtonItemStylePlain target:self action:@selector(logout:)];
    
    //Setup the refresh button
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(refreshUserData:)];
    
    switch (self.socialAccountType) {
        case SocialAccountTypeTwitter:
            self.statusLabel.text = @"Last tweet:";
            self.socialLogoImageView.image = [UIImage imageNamed:@"twitterIcon"];
            break;
        case SocialAccountTypeFacebook:
            self.statusLabel.text = @"Last update:";
            self.socialLogoImageView.image = [UIImage imageNamed:@"facebookIcon"];
            break;
        default:
            break;
    }
    [self displayUserDetails];
}


#pragma mark Other
- (void)displayUserDetails {
    //If the loading of the user details has finished.
    if(self.userDataLoaded && self.timelineDataLoaded) {
        self.navigationItem.title = self.username; 
        self.nameLabel.text = self.name;
        self.statusTextView.text = self.status;
        self.userPhotoView.frame = CGRectMake(0.0, 0.0, 73.0, 73.0);
        [self.userPhotoContainerView addSubview:self.userPhotoView];
        [self.view setNeedsDisplay];
        [SVProgressHUD dismiss];
    } else {
        //If the loading has not yet finished, try again after a second.
        [self performSelector:@selector(displayUserDetails) withObject:self afterDelay:1.0];
    }
}

- (void)displayErrorMessage {
    NSString *socialNetworkName = nil;
    switch (self.socialAccountType) {
        case SocialAccountTypeTwitter:
            socialNetworkName = @"Twitter";
            break;
        case SocialAccountTypeFacebook:
            socialNetworkName = @"Facebook";
            break;
        default:
            break;
    }
    
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:[NSString stringWithFormat:@"%@ error", socialNetworkName]
                                                    message:[NSString stringWithFormat:@"There was an error talking to %@. Please try again later.", socialNetworkName]
                                                   delegate:nil
                                          cancelButtonTitle:@"OK"
                                          otherButtonTitles:nil];
    [alert show];
}

#pragma mark IBActions
- (IBAction)logout:(id)sender {
    if(self.socialAccountType == SocialAccountTypeTwitter) {
        
    } else if(self.socialAccountType == SocialAccountTypeFacebook) {
        AppDelegate *appDelegate = [UIApplication sharedApplication].delegate;
        [appDelegate closeFacebookSession];
    }
    //Delete the stored SocialAccountType so next time the app starts, the user will be presented with the login screen.
    [[NSUserDefaults standardUserDefaults] removeObjectForKey:kSocialAccountTypeKey];
    [[NSUserDefaults standardUserDefaults] synchronize];
    [self.navigationController popToRootViewControllerAnimated:YES];
}

- (IBAction)refreshUserData:(id)sender {
    [SVProgressHUD showWithStatus:@"Loading..."];
    self.userDataLoaded = NO;
    self.timelineDataLoaded = NO;
    [self loadUserDetails];
    [self displayUserDetails];
}

#pragma mark Account Data
- (void)loadUserDetails {
    //If the user is logged in using Twitter
    if(self.socialAccountType == SocialAccountTypeTwitter) {
        //Get a reference to the application delegate.
        AppDelegate *appDelegate = [UIApplication sharedApplication].delegate;
        //Get a reference to the Twitter account stored on the device.
        [appDelegate getTwitterAccountOnCompletion:^(ACAccount *twitterAccount){
            if(twitterAccount) {
                NSURL *credentialsUrl = [NSURL URLWithString:@"http://api.twitter.com/1/account/verify_credentials.json"];
                TWRequest *credentialsRequest = [[TWRequest alloc] initWithURL:credentialsUrl
                                                         parameters:nil
                                                      requestMethod:TWRequestMethodGET];
                
                //Be sure to attach the Twitter ACAccount object to the request.
                credentialsRequest.account = twitterAccount;
                [credentialsRequest performRequestWithHandler:^(NSData *responseData,
                                                     NSHTTPURLResponse *urlResponse,
                                                     NSError *error) {
                    //If there is an error, display a message to the user.
                    if(error != nil) {
                        [self displayErrorMessage];
                        return;
                    }
                    
                    // Parse the JSON response
                    NSError *jsonError = nil;
                    id response = [NSJSONSerialization JSONObjectWithData:responseData
                                                                  options:0
                                                                    error:&jsonError];
                    
                    //If the JSON parser has thrown an error, display a message to the user.
                    if(jsonError != nil) {
                        [self displayErrorMessage];
                        return;
                    }
                    
                    NSString *name = [response objectForKey:@"name"];
                    NSString *screenName = [response objectForKey:@"screen_name"];
                    
                    //Retrieve the users profile image.
                    NSURL *url = [NSURL URLWithString:[NSString stringWithFormat:@"http://api.twitter.com/1/users/profile_image/?screen_name=%@&size=bigger", screenName]];
                    NSData *data = [NSData dataWithContentsOfURL:url];
                    UIImage *img = [[UIImage alloc] initWithData:data];
                    
                    //Store the loaded data in the properties
                    self.name = name;
                    self.username = screenName;
                    self.userPhotoView = [[UIImageView alloc] initWithImage:img];
                    self.userDataLoaded = YES;
                }];
                
                //Retrieve the users tweets
                NSURL *timelineUrl = [NSURL URLWithString:@"http://api.twitter.com/1/statuses/user_timeline.json"];
                TWRequest *timelineRequest = [[TWRequest alloc] initWithURL:timelineUrl
                                                         parameters:nil
                                                      requestMethod:TWRequestMethodGET];
                
                //Be sure to attach the Twitter ACAccount object to the request.
                timelineRequest.account = twitterAccount;
                [timelineRequest performRequestWithHandler:^(NSData *responseData,
                                                     NSHTTPURLResponse *urlResponse,
                                                     NSError *error) {
                    //If there is an error, display a message to the user.
                    if(error != nil) {
                        [self displayErrorMessage];
                        return;
                    }
                    
                    // Parse the JSON response
                    NSError *jsonError = nil;
                    id response = [NSJSONSerialization JSONObjectWithData:responseData
                                                                  options:0
                                                                    error:&jsonError];
                    
                    //If the JSON parser has thrown an error, display a message to the user.
                    if(jsonError != nil){
                        [self displayErrorMessage];
                        return;
                    }
                    
                    //Get the last tweet of the user.
                    NSString *lastTweetText = [[response objectAtIndex:0] objectForKey:@"text"];
                    self.status = lastTweetText;
                    self.timelineDataLoaded = YES;
                }];
            }
        }];
    } else if(self.socialAccountType == SocialAccountTypeFacebook){
        //If the user is logged in using Facebook
        
        //Check whether the current Facebook session is open.
        if([FBSession.activeSession isOpen]) {
            //Request the users information.
            [[FBRequest requestForMe] startWithCompletionHandler:
             ^(FBRequestConnection *connection, 
               NSDictionary<FBGraphUser> *user, 
               NSError *error) {
                 if (!error) {
                     self.name = user.name;
                     self.username = user.username;
                     FBProfilePictureView *fbProfilePictureView = [[FBProfilePictureView alloc] initWithProfileID:user.id pictureCropping:FBProfilePictureCroppingSquare];
                     self.userPhotoView = fbProfilePictureView;
                 } else {
                     [self displayErrorMessage];
                 }
                 self.userDataLoaded = YES;
             }]; 
            //Request the users wall feed.
            [[FBRequest requestForGraphPath:@"me/feed"] startWithCompletionHandler:
             ^(FBRequestConnection *connection, 
               NSDictionary *data, 
               NSError *error) {
                 if (!error) {
                     //Get the last wall message of the user.
                     self.status = [[[data objectForKey:@"data"] objectAtIndex:0] objectForKey:@"message"];
                 } else {
                     [self displayErrorMessage];
                 }
                 //Store the loaded data in the properties
                 self.timelineDataLoaded = YES;
             }];
        }
    }
}

@end
